/*
 * Struts Action Plug-in Extension.
 * Copyright (C) 2002 ASQdotCOM NV (http://www.asqdotcom.be/)
 *
 * This extension is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * This extension is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
package be.ff.gui.web.struts.action;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/**
 * This class represents the values as they are specified in an <code>&lt;action-plug-in&gt;</code> tag of the
 * <code>action-plug-in-config.xml</code> configuration file.
 * 
 * <p>
 * The actual parsing of the <code>action-plug-in-config.xml</code> configuration file is done in the
 * <code>ActionPlugInPlugIn</code> class.
 * </p>
 *
 * @author Gerrie Kimpen
 *
 * @see ActionPlugInChainDefinition
 * @see ActionPlugInPlugIn
 * @since FF 1.0
 */
public class ActionPlugInDefinition
{
    private String className = null;
    private Map initParams = null;
    private Set disabledActionPaths = null;
    private Set disabledActionPathsCuringa = null;
    private Set enabledActionPaths = null;
    private Set enabledActionPathsCuringa = null;
    private Set disabledModules = null;
    private Set enabledModules = null;

    /**
     * Constructs a new <code>ActionPlugInDefinition</code> instance.
     */
    public ActionPlugInDefinition()
    {
        initParams = new HashMap();
        disabledActionPaths = new HashSet();
        disabledActionPathsCuringa = new HashSet();
        enabledActionPaths = new HashSet();
        enabledActionPathsCuringa = new HashSet();
        disabledModules = new HashSet();
        enabledModules = new HashSet();
    }

    /**
     * Returns the value of the <code>&lt;class&gt;</code> tag like it was specified in the action plug-in
     * configuration file.
     *
     * @return the value of the <code>&lt;class&gt;</code> tag
     */
    public String getClassName()
    {
        return className;
    }

    /**
     * Used to set the value of the <code>&lt;class&gt;</code> tag like it was specified in the action plug-in
     * configuration file.
     *
     * @param className the value of the <code>&lt;class&gt;</code> tag
     */
    public void setClassName(String className)
    {
        this.className = className;
    }

    /**
     * Returns a <code>Map</code> that contains the <code>&lt;init-params&gt;</code> like they were specified in the
     * action plug-in configuration file.
     *
     * @return a <code>Map</code> instance that holds key-value pairs represented by the values of the
     *         <code>&lt;name&gt;</code> and <code>&lt;value&gt;</code> tags
     */
    public Map getInitParams()
    {
        return initParams;
    }

    /**
     * Adds a key/value pair to the initialization parameter list that is returned by the <code>getInitParams</code>
     * method.
     *
     * @param name the name of the init parameter, as specified in the <code>&lt;name&gt;</code> tag
     * @param value the value of the init parameter, as specified in the <code>&lt;value&gt;</code> tag
     */
    public void addInitParam(String name, String value)
    {
        initParams.put(name, value);
    }

    /**
     * Returns a <code>Map</code> that contains the (<code>&lt;disabled-for&gt;</code>)
     * <code>&lt;action-path&gt;</code> values, like they were specified in the action plug-in configuration file.
     *
     * @return a <code>Set</code> containing the values of the action paths for which this action plug-in has to be
     *         disabled
     */
    public Set getDisabledActionPaths()
    {
        return disabledActionPaths;
    }

    public Set getDisabledActionPathsCuringa()
    {
        return disabledActionPathsCuringa;
    }

    /**
     * Adds a Struts action path (<code>ActionMapping::getPath</code>) value, which is specified in the action plug-in
     * configuration file, to the <code>Set</code> that is returned by the <code>getDisabledActionPaths</code> method.
     *
     * @param path the value of the action path, as specified in the <code>&lt;action-path&gt;</code> tag
     */
    public void addDisabledActionPath(String path)
    {
        int posicaoCuringa = path.indexOf("*");
        if (posicaoCuringa > 0)
        {
            disabledActionPathsCuringa.add(path.substring(0, posicaoCuringa));

            // Só para garantir que depois do curinga não tem mais nenhum caractere
            if (posicaoCuringa + 1 < path.length())
            {
                throw new RuntimeException("Quando utilizado um curinga (*), este deve ser o último caractere informado.");
            }
        }
        else
        {
            disabledActionPaths.add(path);
        }
    }

    /**
     * Returns <code>true</code> if the specfied action path is in the set of disabled action paths returned by the
     * <code>getDisabledActionPaths</code> method; else <code>false</code>.
     *
     * @param actionPath the Struts action path (<code>ActionMapping::getPath</code>) to search
     *
     * @return <code>true</code> if the specfied action path is in the set of disabled action paths; else
     *         <code>false</code>
     */
    public boolean hasDisabledActionPath(String actionPath)
    {
        boolean retorno = false;

        retorno = contemMapeamento(disabledActionPaths, disabledActionPathsCuringa, actionPath);

        return retorno;
    }

    /**
     * Returns a <code>Map</code> that contains the (<code>&lt;disabled-for&gt;</code>)
     * <code>&lt;action-path&gt;</code> values, like they were specified in the action plug-in configuration file.
     *
     * @return a <code>Set</code> containing the values of the action paths for which this action plug-in has to be
     *         disabled
     */
    public Set getEnabledActionPaths()
    {
        return enabledActionPaths;
    }

    public Set getEnabledActionPathsCuringa()
    {
        return enabledActionPathsCuringa;
    }

    /**
     * Adds a Struts action path (<code>ActionMapping::getPath</code>) value, which is specified in the action plug-in
     * configuration file, to the <code>Set</code> that is returned by the <code>getDisabledActionPaths</code> method.
     *
     * @param path the value of the action path, as specified in the <code>&lt;action-path&gt;</code> tag
     */
    public void addEnabledActionPath(String path)
    {
        int posicaoCuringa = path.indexOf("*");
        if (posicaoCuringa > 0)
        {
            enabledActionPathsCuringa.add(path.substring(0, posicaoCuringa));

            // Só para garantir que depois do curinga não tem mais nenhum caractere
            if (posicaoCuringa + 1 < path.length())
            {
                throw new RuntimeException("Quando utilizado um curinga (*), este deve ser o último caractere informado.");
            }
        }
        else
        {
            enabledActionPaths.add(path);
        }
    }

    /**
     * Returns <code>true</code> if the specfied action path is in the set of disabled action paths returned by the
     * <code>getDisabledActionPaths</code> method; else <code>false</code>.
     *
     * @param actionPath the Struts action path (<code>ActionMapping::getPath</code>) to search
     *
     * @return <code>true</code> if the specfied action path is in the set of disabled action paths; else
     *         <code>false</code>
     */
    public boolean hasEnabledActionPath(String actionPath)
    {
        boolean retorno = false;

        retorno = contemMapeamento(enabledActionPaths, enabledActionPathsCuringa, actionPath);

        return retorno;
    }

    public Set getDisabledModules()
    {
        return disabledModules;
    }

    public void addDisabledModule(String module)
    {
        disabledModules.add(module);
    }

    public boolean hasDisabledModule(String module)
    {
        boolean retorno = false;

        if (disabledModules.size() != 0)
        {
            retorno = contemMapeamento(disabledModules, null, module);
        }

        return retorno;
    }

    public Set getEnabledModules()
    {
        return enabledModules;
    }

    public void addEnabledModule(String module)
    {
        enabledModules.add(module);
    }

    public boolean hasEnabledModule(String module)
    {
        boolean retorno = false;

        if (enabledModules.size() != 0)
        {
            retorno = contemMapeamento(enabledModules, null, module);
        }

        return retorno;
    }

    /**
     * Returns this class represented as a <code>String</code>.
     *
     * @return this class, represented as a <code>String</code>
     */
    public String toString()
    {
        StringBuffer stringRep = new StringBuffer();

        stringRep.append("-- action plug-in class --\n");
        stringRep.append(getClassName());
        stringRep.append("\n");

        stringRep.append("-- Comportamento deste plugin --\n");
        if (isPadraoHabilitado())
        {
            stringRep.append("HABILITADO");
        }
        else
        {
            stringRep.append("DESABILITADO");
        }
        stringRep.append("\n");

        stringRep.append("-- init params (key = value) --\n");
        Set set = getInitParams().keySet();
        Iterator iter = set.iterator();
        while (iter.hasNext())
        {
            String name = (String) iter.next();
            String value = (String) initParams.get(name);
            stringRep.append(name);
            stringRep.append(" = ");
            stringRep.append(value);
            stringRep.append("\n");
        }

        stringRep.append("-- disabled action paths --\n");
        set = getDisabledActionPaths();
        iter = set.iterator();
        while (iter.hasNext())
        {
            String path = (String) iter.next();
            stringRep.append(path);
            stringRep.append("\n");
        }
        set = getDisabledActionPathsCuringa();
        iter = set.iterator();
        while (iter.hasNext())
        {
            String path = (String) iter.next();
            stringRep.append(path);
            stringRep.append("\n");
        }

        stringRep.append("-- enabled action paths --\n");
        set = getEnabledActionPaths();
        iter = set.iterator();
        while (iter.hasNext())
        {
            String path = (String) iter.next();
            stringRep.append(path);
            stringRep.append("\n");
        }
        set = getEnabledActionPathsCuringa();
        iter = set.iterator();
        while (iter.hasNext())
        {
            String path = (String) iter.next();
            stringRep.append(path);
            stringRep.append("\n");
        }

        stringRep.append("-- disabled modules --\n");
        set = getDisabledModules();
        iter = set.iterator();
        while (iter.hasNext())
        {
            String module = (String) iter.next();
            stringRep.append(module);
            stringRep.append("\n");
        }

        stringRep.append("-- enabled modules --\n");
        set = getEnabledModules();
        iter = set.iterator();
        while (iter.hasNext())
        {
            String module = (String) iter.next();
            stringRep.append(module);
            stringRep.append("\n");
        }

        return stringRep.toString();
    }

    private boolean contemMapeamento(Set mapeamento, Set mapeamentoCuringa, String strMapeamento)
    {
        boolean retorno = false;
        if (mapeamento.size() > 0)
        {
            retorno = mapeamento.contains(strMapeamento);
        }

        if (retorno == false && mapeamentoCuringa != null && mapeamentoCuringa.size() > 0)
        {
            Iterator iter = mapeamentoCuringa.iterator();
            while (iter.hasNext() && retorno == false)
            {
                String pathParcial = (String) iter.next();
                retorno = strMapeamento.startsWith(pathParcial);
            }
        }

        return retorno;
    }

    public boolean isPadraoHabilitado()
    {
        // Se tem a cláusula DISABLED-FOR no xml, é porque o padrão é HABILITADO
        return (disabledActionPaths.size() > 0 || disabledModules.size() > 0);
    }
}
