/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.biblioteca.util;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * Esta classe armazena os valores de uma propriedade em diferentes contextos.
 * Por exemplo, a propriedade "paginacao" tem o valor "20" no contexto "janela" e "50" no contexto "pagina".
 * O primeiro valor atribudo a uma propriedade, caso no seja explicitamente alterado posteriormente,
 *  o valor "padro" e ser sempre retornado quando no for especificado o contexto.    
 */
public class ConteinerSingleton
{
    public static final ConteinerSingleton INSTANCIA = new ConteinerSingleton();

    private static final String CONTEXTO_PADRAO = "CONTEXTO#PADRAO";
    private Map<String, Map<String, Object>> mapaPropriedades = new ConcurrentHashMap<String, Map<String, Object>>();

    private ConteinerSingleton()
    {}

    /**
     * Este mtodo "salva" as propriedades em um map de maps. No primeiro map, a key  o 
     * nome da propriedade, e no segundo, a key  o nome do mdulo a que se refere aquela 
     * propriedade. No segundo map, qdo o segundo mdulo  inserido, o valor definido para
     * o primeiro  definido como o valor padro caso se tente obter o valor da propriedade
     * para um mdulo nao definido no map.
     * 
     * @param nomePropriedade Propriedade
     * @param valorObjeto Valor
     */
    public void adicionarPropriedade(String nomePropriedade, Object valorObjeto)
    {
        adicionarPropriedade(nomePropriedade, valorObjeto, CONTEXTO_PADRAO);
    }

    public void adicionarPropriedade(String nomePropriedade, Object valorObjeto, String contexto)
    {
        Map<String, Object> mapaValores = mapaPropriedades.get(nomePropriedade);

        // se ja existe a key
        if (mapaValores == null)
        {
            mapaValores = new ConcurrentHashMap<String, Object>();
            mapaPropriedades.put(nomePropriedade, mapaValores);
        }

        // Se for a primeira entrada, informar que este primeiro valor  tambm o valor padro da propriedade
        if (mapaValores.isEmpty())
        {
            mapaValores.put(CONTEXTO_PADRAO, valorObjeto);
        }
        if (!CONTEXTO_PADRAO.equals(contexto))
        {
            mapaValores.put(contexto, valorObjeto);
        }
    }

    public void removerPropriedade(String nomePropriedade)
    {
        removerPropriedade(nomePropriedade, null);
    }

    public void removerPropriedade(String nomePropriedade, String contexto)
    {
        Map<String, Object> mapaValores = mapaPropriedades.get(nomePropriedade);

        // S faz sentido remover se a propriedade est no conteiner
        if (mapaValores != null)
        {
            if (contexto != null)
            {
                // Como foi especificado o contexto, ento remover somente do contexto em questo
                mapaValores.remove(contexto);
            }

            if (contexto == null || mapaValores.size() <= 1)
            {
                // Se foi pedido para remover de todo os contextos (contexto == nulo) ou no sobrou nenhum contexto ou s o padro,
                // ento remover a propriedade do conteiner
                mapaPropriedades.remove(nomePropriedade);
            }
        }
    }

    /**
     * Obtm valor padro da propriedade.
     * 
     * @param nomePropriedade nome da propriedade
     * @return valor da propriedade para o contexto padro
     */
    public Object obterValorObjeto(String nomePropriedade)
    {
        return obterValorObjeto(nomePropriedade, CONTEXTO_PADRAO);
    }

    /**
     * Obtm o valor da propriedade para o contexto especificado.
     * 
     * @param nomePropriedade nome da propriedade
     * @param contexto nome do conexto para a recuperao do valor
     * @return valor da propriedade para o contexto especificado
     */
    public Object obterValorObjeto(String nomePropriedade, String contexto)
    {
        Object retorno = null;

        Map<String, Object> mapaValores = (Map<String, Object>) mapaPropriedades.get(nomePropriedade);
        if (mapaValores != null)
        {
            retorno = (Object) mapaValores.get(contexto);
        }

        return retorno;
    }

}
