/****
 * Copyright 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Cmara dos Deputados, Brasil
 * 
 * Este arquivo  parte do programa TALENTOS - Banco de Talentos
 *
 * O TALENTOS  um software livre; voc pode redistribu-lo e/ou modific-lo dentro dos termos da Licena Pblica Geral GNU como publicada pela Fundao do Software Livre (FSF); na verso 2 da Licena.
 *
 * Este programa  distribudo na esperana que possa ser til, mas SEM NENHUMA GARANTIA; sem uma garantia implcita de ADEQUAO a qualquer MERCADO ou APLICAO EM PARTICULAR. Veja a Licena Pblica Geral GNU para maiores detalhes.
 * 
 * Voc deve ter recebido uma cpia da Licena Pblica Geral GNU, sob o ttulo "LICENCA.txt", junto com este programa, se no, escreva para a Fundao do Software Livre(FSF) Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***/

package br.gov.camara.negocio;

import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import br.gov.camara.negocio.exception.DAOException;

public class ManipuladorSql
{
    private static final String QUEBRA_DE_LINHA = "\n";

    private StringBuilder buffer;
    private Map<String, String> contexto;
    private ParametroDAO parametros;
    private Map<String, Integer> tiposParametros;

    public ManipuladorSql()
    {
        this("");
    }

    public ManipuladorSql(String sql)
    {
        this.buffer = new StringBuilder(sql);
        this.contexto = new HashMap<String, String>();
        this.parametros = new ParametroDAO();
        this.tiposParametros = new HashMap<String, Integer>();
    }

    public static ManipuladorSql com(String... linhasSql)
    {
        ManipuladorSql manipuladorSql = new ManipuladorSql();
        manipuladorSql.adicionarSql(linhasSql);
        return manipuladorSql;
    }

    public void adicionarSql(String... linhasSql)
    {
        for (String linhaSql : linhasSql)
        {
            adicionarSql(linhaSql);
        }
    }

    public void adicionarSql(String sql)
    {
        buffer.append(sql);
        buffer.append(QUEBRA_DE_LINHA);
    }

    public void adicionarSqlComParametro(String sql, String nomeParametro, Object valorParametro)
    {
        adicionarSql(sql);
        comParametro(nomeParametro, valorParametro);
    }

    public void adicionarSqlComParametro(String sql, String nomeParametro, Object valorParametro, int tipoParametro)
    {
        adicionarSql(sql);
        comParametro(nomeParametro, valorParametro, tipoParametro);
    }

    public ManipuladorSql comVariavel(String nomeVariavel, String valorVariavel)
    {
        contexto.put(nomeVariavel, valorVariavel);
        return this;
    }

    public ManipuladorSql comParametro(String nomeParametro, Object valorParametro)
    {
        try
        {
            parametros.adicionar(nomeParametro, valorParametro);
        }
        catch (DAOException e)
        {
            throw new RuntimeException(e);
        }
        return this;
    }

    public ManipuladorSql comParametro(String nomeParametro, Object valorParametro, int tipoSql)
    {
        comParametro(nomeParametro, valorParametro);
        tiposParametros.put(nomeParametro, tipoSql);
        return this;
    }

    public ManipuladorSql comParametros(ParametroDAO outrosParametros)
    {
        for (Entry<Object, Object> parametro : outrosParametros.entrySet())
        {
            comParametro((String) parametro.getKey(), parametro.getValue());
        }
        return this;
    }

    public String gerarSqlFinal()
    {
        String sql = buffer.toString();
        for (Map.Entry<String, String> variavel : contexto.entrySet())
        {
            String expressaoSubstituicao = "\\$\\{" + variavel.getKey() + "\\}";
            sql = sql.replaceAll(expressaoSubstituicao, variavel.getValue());
        }

        return sql;
    }

    public String gerarSqlFinalComValoresDeParametros()
    {
        String sql = gerarSqlFinal();
        FormatadorParametrosSql formatador = new FormatadorParametrosSql();
        return formatador.aplicarParametros(sql, parametros);
    }

    public ParametroDAO getParametros()
    {
        return parametros;
    }

    public Map<String, Integer> getTiposParametros()
    {
        return Collections.unmodifiableMap(tiposParametros);
    }

    public ManipuladorSql adicionarManipulador(ManipuladorSql outroManipulador)
    {
        buffer.append(outroManipulador.gerarSqlFinal());
        comParametros(outroManipulador.getParametros());

        return this;
    }

    @Override
    public String toString()
    {
        final int maxLen = 10;
        return "{buffer="
                + buffer
                + ", contexto="
                + (contexto != null ? toString(contexto.entrySet(), maxLen) : null)
                + ", parametros="
                + parametros
                + ", tiposParametros="
                + (tiposParametros != null ? toString(tiposParametros.entrySet(), maxLen) : null)
                + "}";
    }

    private String toString(Collection<?> collection, int maxLen)
    {
        StringBuilder builder = new StringBuilder();
        builder.append("[");
        int i = 0;
        for (Iterator<?> iterator = collection.iterator(); iterator.hasNext() && i < maxLen; i++)
        {
            if (i > 0)
                builder.append(", ");
            builder.append(iterator.next());
        }
        builder.append("]");
        return builder.toString();
    }

}

class FormatadorParametrosSql
{
    //TODO: criar subclasses com formatos especificos de cada banco de dados
    private static final String DELIMITADOR_TEXTO_PADRAO = "'";
    private static final String FORMATO_DATA_PADRAO = "dd/MM/yyyy";

    String aplicarParametros(String sql, ParametroDAO parametros)
    {
        for (Map.Entry<Object, Object> parametro : parametros.entrySet())
        {
            String nomeParametro = (String) parametro.getKey();
            String valorParametro = formatar(parametro.getValue());
            sql = sql.replaceAll(":" + nomeParametro, valorParametro);
        }
        return sql;
    }

    String formatar(Object valor)
    {
        if (valor == null)
        {
            return "NULL";
        }
        if (valor instanceof String)
        {
            return formatarTexto((String) valor);
        }
        if (valor instanceof Date)
        {
            return formatarData((Date) valor);
        }
        if (valor instanceof Object[])
        {
            return formatarArray((Object[]) valor);
        }
        if (valor instanceof Iterable)
        {
            return formatarColecao((Iterable) valor);
        }
        return String.valueOf(valor);
    }

    String formatarTexto(String texto)
    {
        return DELIMITADOR_TEXTO_PADRAO + texto + DELIMITADOR_TEXTO_PADRAO;
    }

    String formatarData(Date data)
    {
        SimpleDateFormat formatador = new SimpleDateFormat(FORMATO_DATA_PADRAO);
        String textoData = formatador.format(data);
        String parametroData = formatarTexto(textoData);
        String parametroFormato = formatarTexto(FORMATO_DATA_PADRAO);
        return String.format("TO_DATE(%1$s, %2$s)", parametroData, parametroFormato);
    }

    String formatarColecao(Iterable<Object> valores)
    {
        StringBuilder buffer = new StringBuilder();
        int i = 0;
        for (Object valor : valores)
        {
            if (i++ > 0)
            {
                buffer.append(", ");
            }
            String textoValor = formatar(valor);
            buffer.append(textoValor);
        }
        return buffer.toString();
    }

    String formatarArray(Object[] valores)
    {
        StringBuilder buffer = new StringBuilder();
        for (int i = 0; i < valores.length; i++)
        {
            if (i > 0)
            {
                buffer.append(", ");
            }
            String textoValor = formatar(valores[i]);
            buffer.append(textoValor);
        }
        return buffer.toString();
    }
}